double precision function phint(phi, a, u)

! ******************************************************************
! *                                                                *
! * evaluate the integral from zero to phi of                      *
! *                                                                *
! *                 u                 2                            *
! *  ( ----------------------------- )  dphi                       *
! *                          2     2                               *
! *    (1 - a * u * cos(phi))  +  u                                *
! *                                                                *
! * for 0 <= phi <= pi , u >= 0 and a >= 0                         *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: phi
  double precision, intent(in) :: a
  double precision, intent(in) :: u

  double precision :: ai, ar, bi, br, c, cpr, d, e, esr, pi, qr, ri, rm, root
  double precision :: rp, rr, s, spr, tm, tp, u2, x, zeta, zetai, zetar, zr

  pi = 3.141592653589793238d0
  c = dcos(phi)
  s = dsin(phi)
  u2 = u**2
  e = a*u
  if (u < 1.0d0 .and. e < 1.0d-02 * (1.0d0 + u2)) then
    zr = 1.0d0 + u2
    esr = e / zr
    phint = u2 / zr**2 * ((( (4.0d0 / 3.0d0) * (2.0d0 + c**2) * s * (5.0d0 - 3 * u2) *  &
            esr + (phi + c * s) * (5.0d0 - u2)) * esr + 4 * s) * esr + phi)
  else
    rm = dsqrt((1.0d0 - e)**2 + u2)
    tm = 0.5d0 * datan2(u, 1.0d0 - e)
    rp = dsqrt((1.0d0 + e)**2 + u2)
    tp = 0.5d0 * datan2(u, 1.0d0 + e)
    root = dsqrt(rm * rp)
    cpr = dcos(tm + tp)
    spr = dsin(tm + tp)
    x = 0.0d0       ! ensure initialization
    if (c >= 0.0d0) then
      x = s / (1.0d0 + c)
    elseif (dabs(s) > 1.0d-07) then
      x = (1.0d0 - c) / s
    endif
    if ((c >= 0.0d0) .or. (dabs(s) > 1.0d-07)) then
      zeta = dsqrt(rm / rp)
      zetar = -zeta * dsin(tm - tp)
      zetai =  zeta * dcos(tm - tp)
      br = 0.5d0 * dlog(((zetar + x)**2 + zetai**2) / ((zetar - x)**2 + zetai**2))
      bi = datan2(zetai, zetar + x) - datan2(zetai, zetar - x)
      rr = -(br * spr - bi * cpr) / root
      ri = -(bi * spr + br * cpr) / root
      d = e * s / ((1.0d0 - e * c)**2 + u2)
      ar = d * (1.0d0 - e * c) - rr + u * ri
      ai = -d * u - ri - u * rr
    else
      rr = -pi / root * cpr
      ri =  pi / root * spr
      ar = -rr + u * ri
      ai = -ri - u * rr
    endif
    qr = (ar * (cpr - spr) * (cpr + spr) + 2 * ai * cpr * spr) / (rm * rp)
    phint = 0.5d0 * (ri / u - qr)
  endif
  return
end function phint
