      PROGRAM BOSON                                                     
C    *******************************************************************
C    *                                                                 *
C    * PERFORM THE QUANTUM-MECHANICAL COMPLEMENT TO THE CLASSICAL STEP *
C    * OF THE DIELECTRIC THEORY OF EELS IN SPECULAR GEOMETRY USING A   *
C    * SUITABLE THERMODYNAMICAL AVERAGE OF THE QUANTIZED SURFACE       *
C    * HARMONIC OSCILLATORS                                            *
C    *                                                                 *
C    *******************************************************************
      PARAMETER(MMAX=14,NMAX=2**MMAX)                                   
      LOGICAL PICM,PICP                                                 
      CHARACTER TITLE1*48,TITLE2*72                                     
      DIMENSION P1(NMAX),P2(NMAX),R1(NMAX),R2(NMAX)                     
*                                                                       
*     REMARK : THE TWO ARRAYS R1 AND R2 ARE USED FOR FOURIER            
*     TRANSFORMING THE USER-SUPPLIED INSTRUMENTAL RESPONSE OF THE       
*     SPECTROMETER THAT HAS TO BE CODED IN THE EXTERNAL ROUTINE         
*     RESPON CALLED WHEN THE INPUT PARAMETER GAUSS IS < 0 OR > 1.       
*     WITH 0 <= GAUSS <= 1, R1 AND R2 ARE NOT USED.                     
*                                                                       
C *** RATIONAL APPROXIMATIONS FOR EI(U)*EXP(-U) + E1(U)*EXP(+U)         
C *** IN THE INTERVALS (0,1.3) AND (1.3,INFINITY) <ACCURACY : 4.E-04>   
C *** USED FOR FOURIER TRANSFORMING HALF-LORENTZIAN FUNCTIONS           
      O1(U) = -SINH(U)*LOG(U**2) +                                      
     1         U*((0.03114*U**2 + 0.41666)*U**2 + 0.84557)              
      O2(U) = (((202.91/U**2 + 932.21)/U**2 + 41.740)/U**2 + 2.0)/      
     2        (((540.88/U**2 + 345.67)/U**2 + 18.961)/U**2 + 1.0)/U     
C                                                                       
      DATA FM1/0.0/,FP1/0.0/                                            
      PI = 4.0*ATAN(1.0)                                                
C                                                                       
C *** READ INPUT PARAMETERS                                             
C                                                                       
      call change_working_dir()
      OPEN(UNIT=13,FILE ='BOSIN')                                       
C     TARGET TEMPERATURE (KELVIN)                                       
      READ(13,*) T                                                      
C     WIDTH OF THE INSTRUMENTAL RESPONSE (CM**-1)                       
      READ(13,*) WIDTH                                                  
C     FRACTION OF GAUSSIAN FOR THE INSTRUMENTAL RESPONSE                
      READ(13,*) GAUSS                                                  
C     ASYMMETRY OF THE INSTRUMENTAL RESPONSE                            
      READ(13,*) ASYM                                                   
C     LOWER AND UPPER ENERGY LOSSES FOR THIS COMPUTATION (CM**-1)       
      READ(13,*) EMIN                                                   
      READ(13,*) EMAX                                                   
      CLOSE(UNIT=13)                                                    
      WRITE(6,101) T,WIDTH,GAUSS,ASYM,EMIN,EMAX                         
C                                                                       
C *** READ THE TABLE OF VALUES OF THE CLASSICAL LOSS SPECTRUM           
C                                                                       
      OPEN(UNIT=12,FILE='EELSOU')                                       
      READ(12,'(A48)') TITLE1                                           
      READ(12,102) TITLE2                                               
      N = 0                                                             
    1 READ(12,*,END=2) WMAX,Y                                           
      IF(WMAX.LT.0.0) GOTO 1                                            
      N = N+1                                                           
      P2(N) = Y                                                         
      IF(N.EQ.1) WMIN = WMAX                                            
      IF(N.LT.NMAX) GOTO 1                                              
    2 CLOSE(UNIT=12)                                                    
      IF(N.LE.0) STOP '*** NO INPUT VALUES FOR PCL ***'                 
      WRITE(6,103) TITLE2,N,WMIN,WMAX                                   
      DWN = (WMAX-WMIN)/(N-1)                                           
C                                                                       
C *** REDEFINE THE FREQUENCY INTERVAL (0,WMAX) TO BE USED FOR THE       
C *** FOURIER TRANSFORMS                                                
C                                                                       
      JMIN = INT(0.5+WMIN/DWN)                                          
      IF((JMIN-0.5)*DWN.LT.WMIN) JMIN = JMIN+1                          
      FAC = ((JMIN-0.5)*DWN-WMIN)/DWN                                   
      JMAX = INT(0.5+WMAX/DWN)                                          
      TEST = MAX(1.5*ABS(EMIN),1.5*ABS(EMAX),6.0*WMAX)                  
      N = 2                                                             
      DO 3 M=2,MMAX                                                     
         N = 2*N                                                        
         WMAX = N*DWN                                                   
         IF(WMAX.GE.TEST) GOTO 4                                        
    3 CONTINUE                                                          
      M = MMAX                                                          
      N = NMAX                                                          
      WMAX = N*DWN                                                      
      WRITE(6,104) NMAX,MMAX                                            
    4 WRITE(6,105) WMAX,DWN,N,M                                         
      DO 5 J=1,N                                                        
         P1(J) = 0.0                                                    
    5 CONTINUE                                                          
C                                                                       
C *** INTERPOLATE PCL ON A SUITABLE MESH IN (0,WMAX)                    
C                                                                       
      I = 1                                                             
      DO 6 J=JMIN,MIN(JMAX,N)                                           
         P1(J) = P2(I)+FAC*(P2(I+1)-P2(I))                              
         I = I+1                                                        
    6 CONTINUE                                                          
      DO 7 J=1,N                                                        
         P2(J) = P1(J)                                                  
    7 CONTINUE                                                          
C                                                                       
C *** CHARACTERISTIC FUNCTION F(TAU)                                    
C                                                                       
      CALL SICOT(P1,M,DWN,1.39*T)                                       
      CALL SINTR(P2,M,DWN)                                              
      H = (PI+PI)/WMAX                                                  
      IF(GAUSS.LT.0 .OR. GAUSS.GT.1.0) GOTO 12                          
C                                                                       
C *** BROADEN THE SPECTRUM BY CONVOLUTING THE CHARACTERISTIC FUNCTION BY
C *** A WEIGHTED SUM OF A LORENTZIAN AND A GAUSSIAN RESPONSE FUNCTIONS  
C                                                                       
      ALFA = MAX(1.5*DWN,WIDTH)                                         
      IF(ALFA.GT.WIDTH) WRITE(6,106) ALFA                               
      IF(ALFA.GT.0.5*WMAX/PI) STOP                                      
     .       '*** WIDTH IS TOO LARGE, NOTHING DONE ***'                 
      ALFA = 0.5*ALFA                                                   
      SIGMA = ALFA/1.66511                                              
      A1 = (1.0-ASYM)/2.0*(1.0-GAUSS)                                   
      A2 = (1.0+ASYM)/2.0*(1.0-GAUSS)                                   
      IF(A1.LT.0.0 .OR. A2.LT.0.0) STOP                                 
     .       '*** INVALID INPUT : ASYM SHOULD BE IN (-1,+1) ***'        
      G1 = (1.0-ASYM)*ALFA                                              
      G2 = (1.0+ASYM)*ALFA                                              
      P1(1) = 1.0                                                       
      P2(1) = 0.0                                                       
      DO 11 I=2,N                                                       
         X = (I-1)*H                                                    
         FR = 0.0                                                       
         FI = 0.0                                                       
         IF(A1.EQ.0.0) GOTO 8                                           
         X1 = G1*X                                                      
         IF(X1.LE.100.0) FR = A1*EXP(-X1)                               
         IF(A1.EQ.A2) GOTO 8                                            
         IF(X1.LE.1.3)   FI = A1*O1(X1)                                 
         IF(X1.GT.1.3)   FI = A1*O2(X1)                                 
    8    IF(A2.EQ.0.0) GOTO 9                                           
         X2 = G2*X                                                      
         IF(X2.LE.100.0) FR = FR+A2*EXP(-X2)                            
         IF(A2.EQ.A1) GOTO 9                                            
         IF(X2.LE.1.3)   FI = FI-A2*O1(X2)                              
         IF(X2.GT.1.3)   FI = FI-A2*O2(X2)                              
    9    IF(GAUSS.EQ.0.0) GOTO 10                                       
         X3 = SIGMA*X                                                   
         IF(X3.LE.10.0) FR=FR+GAUSS*EXP(-X3**2)                         
   10    FI = FI/PI                                                     
         FAC = EXP(-P1(I))                                              
         CP2 = FAC*COS(P2(I))                                           
         SP2 = FAC*SIN(P2(I))                                           
         P1(I) = FR*CP2+FI*SP2                                          
         P2(I) = FI*CP2-FR*SP2                                          
   11 CONTINUE                                                          
      IF(ABS(FI).GT.1.0E-03) WRITE(6,107) DWN                           
      GOTO 15                                                           
C                                                                       
C *** BROADEN THE SPECTRUM BY CONVOLUTING THE CHARACTERISTIC FUNCTION   
C *** WITH A USER-SUPPLIED RESPONSE FUNCTION (ARBITRARY NORMALIZATION)  
C                                                                       
   12 WRITE(6,*) '==> SWITCH TO A USER-SUPPLIED INSTRUMENTAL RESPONSE'  
      ALFA = AMAX1(4.0*DWN,WIDTH)                                       
      IF(ALFA.GT.WIDTH) WRITE(6,106) ALFA                               
      IF(ALFA.LT.10.0*DWN) WRITE(6,107) DWN                             
C     MAKE A TABLE OF THE RESPONSE FUNCTION                             
      R1(1) = RESPON(0.0,ALFA)                                          
      R2(1) = R1(1)                                                     
      DO 13 I=2,N                                                       
         X = (I-1)*DWN                                                  
         R1(I) = RESPON(X,ALFA)                                         
         R2(I) = RESPON(-X,ALFA)                                        
   13 CONTINUE                                                          
C     FOURIER TRANSFORM IT                                              
      CALL RCFFI(R1,R2,-M,DWN)                                          
C     NORMALIZATION OF THE RESPONSE FUNCTION, AND CONVOLUTION           
      ANORM = R1(1)                                                     
      DO 14 I=1,N                                                       
         FAC = EXP(-P1(I))/ANORM                                        
         CP2 = FAC*COS(P2(I))                                           
         SP2 = FAC*SIN(P2(I))                                           
         P1(I) = R1(I)*CP2+R2(I)*SP2                                    
         P2(I) = R2(I)*CP2-R1(I)*SP2                                    
   14 CONTINUE                                                          
      ALFA = ALFA/2.0                                                   
C                                                                       
C *** FULL EELS SPECTRUM                                                
C                                                                       
   15 CALL RCFFI(P1,P2,M,H)                                             
C                                                                       
C *** OUTPUT                                                            
C                                                                       
      OPEN(UNIT=14,FILE='BOSOU')                                        
      ISTEP = MAX(INT(ALFA/DWN/10.0),1)                                 
      WRITE(14,108) TITLE1,T,GAUSS                                      
      WRITE(14,102) TITLE2                                              
      NOUT = 0                                                          
      IF(EMIN.GT.0.0) GOTO 18                                           
      IMIN = N-INT(ABS(EMIN)/DWN)                                       
      IF((IMIN-N)*DWN.LT.EMIN) IMIN = IMIN+1                            
      DO 17 I=IMIN,N                                                    
         J = N-I                                                        
         IF(MOD(J,ISTEP).GT.0) GOTO 17                                  
         X = -J*DWN                                                     
         IF(X.GT.EMAX) GOTO 20                                          
         NOUT = NOUT+1                                                  
         WRITE(14,109) X,P2(J+1)                                        
   17 CONTINUE                                                          
   18 IF(EMAX.LT.DWN) GOTO 20                                           
      IMAX = INT(EMAX/DWN)+1                                            
      IF((IMAX-1)*DWN.GT.EMAX) IMAX = IMAX-1                            
      IF(IMAX.LT.2) GOTO 20                                             
      DO 19 I=2,IMAX                                                    
         IF(MOD(I-1,ISTEP).GT.0) GOTO 19                                
         X = (I-1)*DWN                                                  
         IF(X.LT.EMIN) GOTO 19                                          
         NOUT = NOUT+1                                                  
         WRITE(14,109) X,P1(I)                                          
   19 CONTINUE                                                          
   20 CLOSE(UNIT=14)                                                    
      WRITE(6,110) NOUT,ISTEP*DWN                                       
C                                                                       
C *** ANALYZE THE SPECTRUM                                              
C                                                                       
      WRITE(6,111)                                                      
      WN = 0.0                                                          
      FM = P1(1)                                                        
      FP = P2(1)                                                        
      IF(P2(2).LT.FP .AND. P1(2).LT.FP) WRITE(6,112) WN,FP              
      FAC = 5.0E-06*FP*DWN                                              
      IMAX = 2+INT(AMAX1(ABS(EMIN),ABS(EMAX))/DWN)                      
      DO 25 I=2,IMAX                                                    
         FM0 = FM1                                                      
         FP0 = FP1                                                      
         FM1 = FM                                                       
         FP1 = FP                                                       
         FM = P2(I)                                                     
         FP = P1(I)                                                     
         IF(I.EQ.2) GOTO 25                                             
         PICM = .FALSE.                                                 
         PICP = .FALSE.                                                 
         WN = (I-1)*DWN                                                 
         IF(FM1-FM0.LT.0.0) GOTO 22                                     
         IF(FM1-FM.LT.0.0) GOTO 22                                      
         A = (FM1-FM0)+(FM1-FM)                                         
         IF(A.LT.FAC) GOTO 22                                           
         B = 0.5*((FM1-FM0)+3.0*(FM1-FM))                               
         U = B/A                                                        
         WMPIC = -WN+U*DWN                                              
         FMPIC = FM+0.5*B*U                                             
         PICM = .TRUE.                                                  
   22    IF(FP1-FP0.LT.0.0) GOTO 23                                     
         IF(FP1-FP.LT.0.0) GOTO 23                                      
         A = (FP1-FP0)+(FP1-FP)                                         
         IF(A.LT.FAC) GOTO 23                                           
         B = 0.5*((FP1-FP0)+3.0*(FP1-FP))                               
         U = B/A                                                        
         WPPIC = WN-U*DWN                                               
         FPPIC = FP+0.5*B*U                                             
         PICP = .TRUE.                                                  
         IF(PICP.AND.PICM) WRITE(6,112) WPPIC,FPPIC,WMPIC,FMPIC         
         IF(PICP.AND.(.NOT.PICM)) WRITE(6,112) WPPIC,FPPIC              
   23    IF(PICM.AND.(.NOT.PICP)) WRITE(6,113) WMPIC,FMPIC              
   25 CONTINUE                                                          
      STOP                                                              
  101 FORMAT(' PROGRAM BOSON (MARCH 1990)'/' T =',F6.1,' K ,  WIDTH =', 
     ,       F7.2,' CM**-1'/' GAUSS =',F5.2,' ,  ASYM =',F5.2/          
     .       ' ENERGY LOSSES FROM',G11.4,'  TO',G11.4,' CM**-1')        
  102 FORMAT(A72)                                                       
  103 FORMAT(1X,A72/' READ',I6,' VALUES OF PCL FROM',G15.7,' TO',G15.7) 
  104 FORMAT(' +++ N HAS BEEN FIXED AT',I6,' = 2**',I2,' +++')          
  105 FORMAT(' CLASSICAL SPECTRUM REDEFINED FROM 0.0 TO',G14.7/         
     ,' STEP SIZE =',G10.3,I10,' (= 2**',I2,') POINTS')                 
  106 FORMAT(' +++ WIDTH HAS BEEN ENLARGED TO',G11.4,' +++')            
  107 FORMAT(' ... POOR REPRESENTATION OF THE RESPONSE FUNCTION ...'/5X,
     ,'THE STEP SIZE (',G10.3,') SHOULD BE REDUCED')                    
  108 FORMAT(A48,'T =',F6.1,' GAUSS =',F5.2)                            
  109 FORMAT(2E15.7)                                                    
  110 FORMAT(/1X,I6,' VALUES WRITTEN ON DISK, STEP SIZE =',G10.3)       
  111 FORMAT(/2(' PEAK LOCATION    AMPLITUDE',5X))                      
  112 FORMAT(2(1X,F13.2,E13.4,5X))                                      
  113 FORMAT(33X,F13.2,E13.4)                                           
      END                                                               
