      DOUBLE PRECISION FUNCTION SURLOS(DK)                              
C    ****************************************************************** 
C    *                                                                * 
C    * EELS SURFACE LOSS FUNCTION FOR AN ARBITRARY MULTILAYERED TARGET* 
C    *                                                                * 
C    ****************************************************************** 
      PARAMETER(LMAX=100)                                               
      LOGICAL STATIC,ZERO                                               
      DOUBLE PRECISION ARG,ARGMIN,ARGMAX,CN,CNM1,D,DK,EPSMAC,SN,SNM1,TN 
      COMPLEX A,B,CSI,EPS,PNM2,PNM1,PN,PP,QNM2,QNM1,QN,QP,Z             
      DIMENSION ARG(LMAX)                                               
      COMMON/MULAYR/EPS(LMAX),D(LMAX),ARGMIN,ARGMAX,EPSMAC,LAYERS,NPER  
      ZERO(Z) = REAL(Z).EQ.0.0 .AND. AIMAG(Z).EQ.0.0                    
      LSTART = LAYERS-NPER+1                                            
      STATIC = .TRUE.                                                   
      N = 1                                                             
    1 ARG(N) = DK*D(N)                                                  
      IF(ARG(N).GT.ARGMAX .OR. ZERO(EPS(N))) GOTO 10                    
      IF(N.GE.LSTART .AND. ARG(N).GT.ARGMIN) STATIC = .FALSE.           
      N = N+1                                                           
      IF(N.LE.LAYERS) GOTO 1                                            
C                                                                       
C *** PERIODIC CONTINUED FRACTION, PERIOD = NPER                        
C                                                                       
      IF(NPER.GT.1) GOTO 2                                              
      CSI = EPS(LAYERS)                                                 
      GOTO 9                                                            
    2 IF(STATIC) GOTO 5                                                 
      CN = COSH(ARG(LSTART))                                            
      SN = SINH(ARG(LSTART))                                            
      PNM1 = 1.0                                                        
      PN =  CN                                                          
      PP = EPS(LSTART)*SN                                               
      QNM1 = 0.0                                                        
      QN = SN/EPS(LSTART)                                               
      QP = PN                                                           
      DO 3 N=LSTART+1,LAYERS                                            
         CNM1 = CN                                                      
         SNM1 = SN                                                      
         CN = COSH(ARG(N))                                              
         SN = SINH(ARG(N))                                              
         A = EPS(N)*SN                                                  
         PP = CN*PP+A*PN                                                
         QP = CN*QP+A*QN                                                
         B = (EPS(N-1)/EPS(N))*(SN/SNM1)                                
         A = CNM1*B+CN                                                  
         PNM2 = PNM1                                                    
         PNM1 = PN                                                      
         QNM2 = QNM1                                                    
         QNM1 = QN                                                      
         PN = A*PNM1-B*PNM2                                             
         QN = A*QNM1-B*QNM2                                             
    3 CONTINUE                                                          
      IF(ZERO(QN)) GOTO 4                                               
      A = 0.5*(PN-QP)/QN                                                
      B = SQRT(A**2+PP/QN)                                              
      PN = A-PN/QN                                                      
      IF(ABS(PN+B).GT.ABS(PN-B)) B = -B                                 
      CSI = A+B                                                         
      GOTO 9                                                            
    4 A = QP-PN                                                         
      IF(ZERO(A)) GOTO 12                                               
      CSI = PP/A                                                        
      GOTO 9                                                            
C                                                                       
C *** SMALL-DK LIMIT OF THE PERIODIC TAIL                               
C                                                                       
    5 PN = 0.0                                                          
      QN = 0.0                                                          
      DO 6 N=LSTART,LAYERS                                              
        PN = PN+D(N)*EPS(N)                                             
        QN = QN+D(N)/EPS(N)                                             
    6 CONTINUE                                                          
      IF(ZERO(QN)) GOTO 12                                              
      CSI = SQRT(PN/QN)                                                 
      IF(AIMAG(CSI).GT.0.0) GOTO 9                                      
      IF(AIMAG(CSI).LT.0.0) THEN                                        
         CSI = -CSI                                                     
      ELSE                                                              
         IF(REAL(QN).LT.0.0) CSI = -CSI                                 
      ENDIF                                                             
    9 N = LSTART                                                        
      GOTO 11                                                           
   10 CSI = EPS(N)                                                      
C                                                                       
C *** BACKWARD ALGORITHM                                                
C                                                                       
   11 N = N-1                                                           
      IF(N.LE.0) GOTO 15                                                
      IF(ARG(N).EQ.0.0D0) GOTO 11                                       
      TN = TANH(ARG(N))                                                  
      B = EPS(N)+CSI*TN                                                  
      IF(ZERO(B)) GOTO 13                                               
      CSI = EPS(N)*(CSI+TN*EPS(N))/B                                     
      GOTO 11                                                           
   12 N = LSTART                                                        
   13 IF(N.LE.1) GOTO 14                                                
      N = N-1                                                           
      CSI = EPS(N)/TANH(ARG(N))                                         
      GOTO 11                                                           
   14 SURLOS = 0.0D0                                                    
      GOTO 17                                                           
   15 A = CSI+1.0                                                       
      IF(ZERO(A)) GOTO 16                                               
      SURLOS = AIMAG(-2.0/A)                                            
      GOTO 17                                                           
   16 SURLOS = 2.0D0/EPSMAC                                             
   17 RETURN
      END                                                               
